/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 *
 */
package kr.co.uracle.MPushPlugin.mpush;

import static kr.co.uracle.MPushPlugin.push.PushUiHandler.setCallback;

import android.app.Activity;
import android.os.Handler;
import android.util.Log;

import org.apache.cordova.CallbackContext;
import org.apache.cordova.CordovaPlugin;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import m.client.push.library.PushManager;
import m.client.push.library.common.PushConstants;
import m.client.push.library.utils.PushUtils;

public class MPushPlugin extends CordovaPlugin {
    private static final String TAG = "MPushPlugin";

    /**
     * Executes the request and returns PluginResult.
     *
     * @param action          The action to execute.
     * @param args            JSONArry of arguments for the plugin.
     * @param callbackContext The callback id used when calling back into JavaScript.
     * @return True if the action was valid, false otherwise.
     */

    @Override
    public boolean execute(final String action, JSONArray args, final CallbackContext callbackContext) throws JSONException {
        Log.i(TAG, "Executing action: [" + action + "]");
        final Activity activity = cordova.getActivity();
        PushManager.getInstance().setCurrentCallback(this.cordova.getActivity(),callbackContext.getCallbackId());

        final JSONObject options = args.getJSONObject(0);
        if ("registerServiceAndUser".equals(action)) {
            final JSONObject params = new JSONObject();
            try {
                params.put(PushConstants.KEY_CUID, options.getString("cuid"));
                params.put(PushConstants.KEY_CNAME, options.getString("name"));
            } catch (JSONException e) {
                e.printStackTrace();
            }
            if (PushUtils.checkNetwork(activity)) {
                setCallback(callbackContext);
                new Handler().post(() -> {
                    // TODO Auto-generated method stub
                    PushManager.getInstance().registerServiceAndUser(activity, params);
                });
            } else {
                callbackContext.error("network is not connected.");
            }
        } else if("registerUser".equals(action)){
            JSONObject returnValue = new JSONObject();
            returnValue.put("status", "SUCCESS");
            returnValue.put("message", "RegisterServiceAndUser를 사용하시기 바랍니다.");
            callbackContext.success(returnValue);
        } else if("registerService".equals(action)){
            JSONObject returnValue = new JSONObject();
            returnValue.put("status", "SUCCESS");
            returnValue.put("message", "RegisterServiceAndUser를 사용하시기 바랍니다.");
            callbackContext.success(returnValue);
        }else if ("unRegisterService".equals(action)) {
            if (PushUtils.checkNetwork(activity)) {
                setCallback(callbackContext);
                new Handler().post(() -> {
                    // TODO Auto-generated method stub
                    PushManager.getInstance().unregisterPushService(activity.getApplicationContext());
                });
            } else {
                callbackContext.error("network is not connected.");
            }
        } else if ("unRegisterUser".equals(action)) {
            final JSONObject params = new JSONObject();
            try {
                params.put(PushConstants.KEY_CUID, options.getString("cuid"));
                params.put(PushConstants.KEY_CNAME, options.getString("name"));
            } catch (JSONException e) {
                e.printStackTrace();
            }
            if (PushUtils.checkNetwork(activity)) {
                setCallback(callbackContext);
                new Handler().post(() -> {
                    // TODO Auto-generated method stub
                    PushManager.getInstance().unregisterPushUser(activity.getApplicationContext(), params);
                });
            } else {
                callbackContext.error("network is not connected.");
            }
        } else if ("sendNotification".equals(action)) {
            String ext = "메세지 테스트";
            if (PushUtils.checkNetwork(activity)) {
                setCallback(callbackContext);
                final String finalMessage = options.getString("message");
                final String cuid = options.getString("cuid");
                final String serviceCode = options.getString("code");

                JSONObject object1 = new JSONObject();
                object1.put("title", "일반 알림");
                object1.put("body", finalMessage);
                new Handler().post(() -> {
                    PushManager.getInstance().sendPushMessage(activity.getApplicationContext(), object1.toString(), ext, "", serviceCode);
                });
            } else {
                callbackContext.error("network is not connected.");
            }
        } else if ("remoteSetBadgeNumber".equals(action)) {
            if (PushUtils.checkNetwork(activity)) {
                setCallback(callbackContext);
                new Handler().post(() -> {
                    PushManager.getInstance().initBadgeNo(activity.getApplicationContext(), "0");
                });
            } else {
                callbackContext.error("network is not connected.");
            }
        }else if ("readNotification".equals(action)){
            if (PushUtils.checkNetwork(activity)) {
                setCallback(callbackContext);
                new Handler().post(() -> {
                    try {
                        PushManager.getInstance().pushMessageReadConfirm(activity.getApplicationContext(), options.getString("notification"));
                    } catch (JSONException e) {
                        throw new RuntimeException(e);
                    }
                });
            } else {
                callbackContext.error("network is not connected.");
            }
        }else if("pushMessageReceiveConfirm".equals(action)){
            if (PushUtils.checkNetwork(activity)) {
                setCallback(callbackContext);
                new Handler().post(() -> {
                    try {
                        PushManager.getInstance().pushMessageReceiveConfirm(activity.getApplicationContext(), options.getString("jsonString"));
                    } catch (JSONException e) {
                        throw new RuntimeException(e);
                    }
                });
            } else {
                callbackContext.error("network is not connected.");
            }
        }else if ("registerGroup".equals(action)){
            setCallback(callbackContext);
            PushManager.getInstance().registerUserGroup(activity.getApplicationContext(), options.getString("groupNum"));
        }else if ("unRegisterGroup".equals(action)){
            setCallback(callbackContext);
            PushManager.getInstance().unregisterUserGroup(activity.getApplicationContext(), options.getString("groupNum"));
        }
        return true;
    }

}
