/********* MPushPlugin.m Cordova Plugin Implementation *******/

#import <Cordova/CDV.h>
#import <MPushLibrary/MPushLibrary.h>
#import "PushReceiver.h"

@interface MPushPlugin : CDVPlugin {
  // Member variables go here.
}

@end

@implementation MPushPlugin

- (void) pluginInitialize
{
    PushReceiver *receiver = [[PushReceiver alloc] initWithPlugin:self];
    [[PushManager defaultManager] initilaizeWithDelegate:receiver];
}

- (NSString *)getInvalidMessage:(NSDictionary *)options withValidClasses:(NSDictionary *)validClasses {
    
    for ( NSString *key in validClasses ) {
        Class validClass = [validClasses objectForKey:key];
        id parameter = [options objectForKey:key];
        
        if ( parameter == nil ) {
            return [NSString stringWithFormat:@"key(%@) is null", key];
        }
        
        if ( ![parameter isKindOfClass:validClass] ) {
            return [NSString stringWithFormat:@"key(%@) is invalid type", key];
        }
        
        if ( [validClass isKindOfClass:[NSString class]] && [[parameter stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceAndNewlineCharacterSet]] isEqualToString:@""]) {
            return [NSString stringWithFormat:@"key(%@) is invalid", key];
        }
        else if ( [validClass isKindOfClass:[NSDictionary class]] && [[parameter allKeys] count] == 0 ) {
            return [NSString stringWithFormat:@"key(%@) is invalid", key];
        }
        else if ( [validClass isKindOfClass:[NSArray class]] && [parameter count] == 0 ) {
            return [NSString stringWithFormat:@"key(%@) is invalid", key];
        }
    }
    
    return nil;
}


- (void)notificationCenterGetBadgeNumber:(CDVInvokedUrlCommand*)command
{
    NSNumber *badgeNumber = [[[PushManager defaultManager] notificationCenter] badgeNumber];
    CDVPluginResult *pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsNSInteger:[badgeNumber integerValue]];
    [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
}

- (void)notificationCenterSetBadgeNumber:(CDVInvokedUrlCommand*)command
{
    NSNumber *badgeNumber = command.arguments[0];
    [[[PushManager defaultManager] notificationCenter] setBadgeNumber:badgeNumber];
    CDVPluginResult *pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsNSInteger:[badgeNumber integerValue]];
    [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
}

- (void)registerService:(CDVInvokedUrlCommand*)command
{
    [[PushManager defaultManager] registerService:nil completionHandler:^(BOOL success) {
        CDVPluginResult* pluginResult = nil;
        if(success)
        {
            pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsDictionary:@{@"status":@"SUCCESS"}];
        }
        else
        {
            pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsDictionary:@{@"status":@"FAIL", @"error": @"register service is fail"}];
        }
        [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
    }];
}

- (void)registerUser:(CDVInvokedUrlCommand*)command
{
    NSString *invalidMessage = [self getInvalidMessage:command.arguments[0] withValidClasses:@{
        @"cuid": [NSString class],
        @"name": [NSString class]
    }];
    
    if ( invalidMessage != nil ) {
        [self.commandDelegate sendPluginResult:[CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsDictionary:@{@"status":@"FAIL", @"error": invalidMessage}] callbackId:command.callbackId];
        return;
    }
    
    NSString *cuid = [command.arguments[0] objectForKey:@"cuid"];
    NSString *cname = [command.arguments[0] objectForKey:@"name"];
    [[PushManager defaultManager] registerUser:nil clientUID:cuid clientName:cname completionHandler:^(BOOL success) {
        CDVPluginResult* pluginResult = nil;
        if(success)
        {
            pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsDictionary:@{@"status":@"SUCCESS"}];
        }
        else
        {
            pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsDictionary:@{@"status":@"FAIL", @"error": @"register user is fail"}];
        }
        [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
    }];
}

- (void)registerServiceAndUser:(CDVInvokedUrlCommand*)command
{
    NSString *invalidMessage = [self getInvalidMessage:command.arguments[0] withValidClasses:@{
        @"cuid": [NSString class],
        @"name": [NSString class]
    }];
    
    if ( invalidMessage != nil ) {
        [self.commandDelegate sendPluginResult:[CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsDictionary:@{@"status":@"FAIL", @"error": invalidMessage}] callbackId:command.callbackId];
        return;
    }

    NSString *cuid = [command.arguments[0] objectForKey:@"cuid"];
    NSString *cname = [command.arguments[0] objectForKey:@"name"];
    [[PushManager defaultManager] registerServiceAndUser:nil clientUID:cuid clientName:cname completionHandler:^(BOOL success) {
        CDVPluginResult* pluginResult = nil;
        if(success)
        {
            pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsDictionary:@{@"status":@"SUCCESS"}];
        }
        else
        {
            pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsDictionary:@{@"status":@"FAIL", @"error": @"register serviceAndUser is fail"}];
        }
        [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
    }];
}

- (void)unRegisterService:(CDVInvokedUrlCommand*)command
{
    [[PushManager defaultManager] unregisterService:nil completionHandler:^(BOOL success) {
        CDVPluginResult* pluginResult = nil;
        if(success)
        {
            pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsDictionary:@{@"status":@"SUCCESS"}];
        }
        else
        {
            pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsDictionary:@{@"status":@"FAIL", @"error": @"unregister service is fail"}];
        }
        [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
    }];
}

- (void)unRegisterUser:(CDVInvokedUrlCommand*)command
{
    [[PushManager defaultManager] unregisterUser:nil completionHandler:^(BOOL success) {
        CDVPluginResult* pluginResult = nil;
        if(success)
        {
            pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsDictionary:@{@"status":@"SUCCESS"}];
        }
        else
        {
            pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsDictionary:@{@"status":@"FAIL", @"error": @"unregister user is fail"}];
        }
        [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
    }];
}


- (void)readNotification:(CDVInvokedUrlCommand*)command
{
    NSDictionary *noti = [command.arguments[0] objectForKey:@"notification"];
    [[PushManager defaultManager] read:nil notification:noti completionHandler:^(BOOL success) {
        CDVPluginResult* pluginResult = nil;
        if(success)
        {
            pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsDictionary:@{@"status":@"SUCCESS"}];
        }
        else
        {
            pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsDictionary:@{@"status":@"FAIL", @"error": @"read api is fail"}];
        }
        [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
    }];
}

- (void)remoteSetBadgeNumber:(CDVInvokedUrlCommand*)command
{
    NSString *invalidMessage = [self getInvalidMessage:command.arguments[0] withValidClasses:@{
        @"badge": [NSNumber class]
    }];
    
    if ( invalidMessage != nil ) {
        [self.commandDelegate sendPluginResult:[CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsDictionary:@{@"status":@"FAIL", @"error": invalidMessage}] callbackId:command.callbackId];
        return;
    }

    NSNumber *badgeNumber = [command.arguments[0] objectForKey:@"badge"];
    [[PushManager defaultManager] update:nil badge:badgeNumber completionHandler:^(BOOL success) {
        CDVPluginResult* pluginResult = nil;
        if(success)
        {
            pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsDictionary:@{@"status":@"SUCCESS"}];
        }
        else
        {
            pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsDictionary:@{@"status":@"FAIL", @"error": @"update api is fail"}];
        }
        [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
    }];
    
}

- (void)sendNotification:(CDVInvokedUrlCommand*)command
{
    NSString *invalidMessage = [self getInvalidMessage:command.arguments[0] withValidClasses:@{
        @"cuid": [NSString class],
        @"code": [NSString class],
        @"message": [NSString class]
    }];
    
    if ( invalidMessage != nil ) {
        [self.commandDelegate sendPluginResult:[CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsDictionary:@{@"status":@"FAIL", @"error": invalidMessage}] callbackId:command.callbackId];
        return;
    }
    
    NSString *cuid = [command.arguments[0] objectForKey:@"cuid"];
    NSString *serviceCode = [command.arguments[0] objectForKey:@"code"];
    NSString *message = [command.arguments[0] objectForKey:@"message"];
    
    [[PushManager defaultManager] send:nil clientUID:cuid message:message serviceCode:serviceCode ext:@"" completionHandler:^(BOOL success) {
        CDVPluginResult* pluginResult = nil;
        if(success)
        {
            pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsDictionary:@{@"status":@"SUCCESS"}];
        }
        else
        {
            pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsDictionary:@{@"status":@"FAIL", @"error": @"sending notification is fail"}];
        }
        [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
    }];
}

- (void)registerGroup:(CDVInvokedUrlCommand*)command
{
    NSString *invalidMessage = [self getInvalidMessage:command.arguments[0] withValidClasses:@{
        @"groupNum": [NSString class]
    }];
    
    if ( invalidMessage != nil ) {
        [self.commandDelegate sendPluginResult:[CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsDictionary:@{@"status":@"FAIL", @"error": invalidMessage}] callbackId:command.callbackId];
        return;
    }

    NSString *groupNum = [command.arguments[0] objectForKey:@"groupNum"];
    [[PushManager defaultManager] registerGroup:nil groupSequenceNumber:groupNum completionHandler:^(BOOL success) {
        CDVPluginResult* pluginResult = nil;
        if(success)
        {
            pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsDictionary:@{@"status":@"SUCCESS"}];
        }
        else
        {
            pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsDictionary:@{@"status":@"FAIL", @"error": @"register group is fail"}];
        }
        [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
    }];
}

- (void)unRegisterGroup:(CDVInvokedUrlCommand*)command
{
    NSString *invalidMessage = [self getInvalidMessage:command.arguments[0] withValidClasses:@{
        @"groupNum": [NSString class]
    }];
    
    if ( invalidMessage != nil ) {
        [self.commandDelegate sendPluginResult:[CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsDictionary:@{@"status":@"FAIL", @"error": invalidMessage}] callbackId:command.callbackId];
        return;
    }

    NSString *groupNum = [command.arguments[0] objectForKey:@"groupNum"];
    [[PushManager defaultManager] unregisterGroup:nil groupSequenceNumber:groupNum completionHandler:^(BOOL success) {
        CDVPluginResult* pluginResult = nil;
        if(success)
        {
            pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsDictionary:@{@"status":@"SUCCESS"}];
        }
        else
        {
            pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsDictionary:@{@"status":@"FAIL", @"error": @"unregister group is fail"}];
        }
        [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
    }];
}


- (void)setEnabled:(CDVInvokedUrlCommand*)command
{
    BOOL enabled = command.arguments[0];
    [[PushManager defaultManager] setEnabled:enabled];
}

@end
