/*
       Licensed to the Apache Software Foundation (ASF) under one
       or more contributor license agreements.  See the NOTICE file
       distributed with this work for additional information
       regarding copyright ownership.  The ASF licenses this file
       to you under the Apache License, Version 2.0 (the
       "License"); you may not use this file except in compliance
       with the License.  You may obtain a copy of the License at

         http://www.apache.org/licenses/LICENSE-2.0

       Unless required by applicable law or agreed to in writing,
       software distributed under the License is distributed on an
       "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
       KIND, either express or implied.  See the License for the
       specific language governing permissions and limitations
       under the License.
 */

/**
 * 푸시 노티피케이션 선택 시 이벤트 처리 관련
 * (해당 영역 적용 후 스크립트 > getNotification()를 사용해 비즈니스 작업이 가능)
 * (스크립트 > onReceiveNotification()를 이용하여 데이터를 받아올 수 있으나 노티 선택과는 무관)
 * onNewIntent: 푸시 수신시 앱이 실행되고 있는 경우 처리
 * onMessage: 푸시 수신시 앱이 종료되어 있는 경우 처리
 * **/

package com.uracle.push.demo;

import android.content.Intent;
import android.os.Bundle;
import android.util.Log;

import org.apache.cordova.*;
import org.json.JSONException;
import org.json.JSONObject;

public class MainActivity extends CordovaActivity
{
    @Override
    public void onCreate(Bundle savedInstanceState)
    {
        super.onCreate(savedInstanceState);

        // enable Cordova apps to be started in the background
        Bundle extras = getIntent().getExtras();
        if (extras != null && extras.getBoolean("cdvStartInBackground", false)) {
            moveTaskToBack(true);
        }

        // Set by <content src="index.html" /> in config.xml
        loadUrl(launchUrl);
    }

    //앱 실행중 notification 선택 이후처리
    @Override
    protected void onNewIntent(Intent intent) {
        super.onNewIntent(intent);
        Log.e("onNewIntent::","onNewIntent");
        String notification = intent.getStringExtra("JSON");
        JSONObject payload = new JSONObject();
        try {
            payload.put("status", "SUCCESS");
            payload.put("payload", notification);
            loadUrl("javascript:getNotification(" + payload.toString() + ")");
        } catch (JSONException e) {

        }
    }

    //앱이 종료되었을 경우 notification 선택 이후처리
    @Override
    public Object onMessage(String id, Object data) {
        Log.e("onmessage::",id);
        if (id.equals("onPageFinished")){
            Intent intent = getIntent();
            String notification = intent.getStringExtra("JSON");
            JSONObject tempJSON = new JSONObject();
            try {
                tempJSON.put("status", intent.getStringExtra("PUSH_STATUS"));
                tempJSON.put("payload", notification);
                loadUrl("javascript:getNotification(" + tempJSON.toString() + ")");
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        return super.onMessage(id, data);
    }

}